import 'package:flutter/material.dart';

///单选分段组件
// ignore: must_be_immutable
class SegmentTextBar extends StatefulWidget {
  //文字数据集
  List<String> datas;

  //选中后的回掉
  Function(int) onSelected;

  //默认选中的数据
  int defaultIndex = 0;

  //默认item的宽度
  double itemWidth;

  //默认item的高度
  double itemHeight;

  SegmentBarType barType = SegmentBarType.round;

  //两个item之间的间隔
  double itemSpace;

  SegmentTextBar(
      {@required this.datas,
      this.onSelected,
      this.defaultIndex = 0,
      this.itemHeight = 30,
      this.itemWidth = 90,
      this.itemSpace = 28}) {
    barType = SegmentBarType.round;
  }

  //圆角
  double radius;

  //字体大小
  double fontSize;

  Color normalTextColor;
  Color activeTextColor;

  //背景色
  Color normalBackgroundColor;
  Color activeBackgroundColor;

  //SegmentTextBar.round的阴影
  BoxShadow activeBoxShadow;
  BoxShadow normalBoxShadow;

  //带有圆角的分段组件
  SegmentTextBar.round(
      {@required this.datas,
      this.onSelected,
      this.normalBackgroundColor,
      this.activeBackgroundColor,
      this.defaultIndex = 0,
      this.itemHeight = 30,
      this.radius = 15,
      this.normalTextColor,
      this.activeTextColor,
      this.itemWidth = 90,
      this.fontSize = 16,
      this.itemSpace = 10,
      this.activeBoxShadow,
      this.normalBoxShadow}) {
    barType = SegmentBarType.round;
  }

  SegmentTextBar.custom();

  double lineHeight;

  Color lineColor;

  ///带有底部线条的分段组件
  SegmentTextBar.line({
    @required this.datas,
    this.onSelected,
    this.defaultIndex = 0,
    this.itemHeight = 35,
    this.itemWidth = 64,
    this.lineColor,
    this.lineHeight = 2,
    this.normalTextColor,
    this.activeTextColor,
    this.fontSize = 16,
    this.itemSpace = 10,
  }) {
    barType = SegmentBarType.line;
  }

  @override
  _SegmentTextBarState createState() => _SegmentTextBarState();
}

class _SegmentTextBarState extends State<SegmentTextBar> {
  //显示的文字列表
  List<String> sdkLists;

  //被选中的item的位置
  int selectItemIndex;

  //默认线的颜色
  Color defaultLineColor = Color(0xFF000000);

  //默认选中和未选中的颜色
  Color defaultActiveTextColor = Colors.white;
  Color defaultNormalTextColor = Color(0xFF9FA4B9);

  get itemWidth => widget.itemWidth;

  get lineColor => widget.lineColor;

  get itemHeight => widget.itemHeight;

  get lineHeight => widget.lineHeight;

  //用户设置的颜色
  get normalTextColor => widget.normalTextColor;

  get activeTextColor => widget.activeTextColor;

  get barType => widget.barType;

  get activeBoxShadow => widget.activeBoxShadow;

  get normalBoxShadow => widget.normalBoxShadow;

  @override
  void initState() {
    super.initState();
    sdkLists = widget.datas;
    selectItemIndex = widget.defaultIndex;
  }

  @override
  Widget build(BuildContext context) {
    return SingleChildScrollView(
      scrollDirection: Axis.horizontal,
      child: Container(
        child: Row(
          children: _buildSegments(sdkLists),
        ),
      ),
    );
  }

  List<Widget> _buildSegments(List<String> datas) {
    List<Widget> items = [];
    if (datas == null || datas.length == 0) {
      items.add(SizedBox.shrink());
      return items;
    }
    for (var i = 0; i < datas.length; i++) {
      items.add(_buildItem(i));
    }
    return items;
  }

  Widget _buildItem(int index) {
    Widget w;
    if (selectItemIndex == index) {
      switch (barType) {
        case SegmentBarType.round:
          w = roundSelectedItem(index);
          break;
        case SegmentBarType.line:
          w = lineSelectedItem(index);
          break;
        default:
          w = roundSelectedItem(index);
          break;
      }
    } else {
      switch (barType) {
        case SegmentBarType.round:
          w = roundDefaultItem(index);
          break;
        case SegmentBarType.line:
          w = lineDefaultItem(index);
          break;
        default:
          w = roundDefaultItem(index);
          break;
      }
    }
    return w;
  }

  void updateSelectIndex(int index) {
    if (selectItemIndex == index) {
      return;
    } else {
      selectItemIndex = index;
      if (widget.onSelected != null) widget.onSelected(selectItemIndex);
      setState(() {});
    }
  }

  Widget roundSelectedItem(int index) {
    return Container(
      margin: EdgeInsets.only(right: widget.itemSpace),
      child: InkWell(
        onTap: () {
          updateSelectIndex(index);
        },
        child: Container(
          decoration: BoxDecoration(
              boxShadow: [activeBoxShadow ?? defaultActiveBoxShadow],
              borderRadius: BorderRadius.circular(widget.radius),
              color: Color(0xFF697FFE)),
          alignment: Alignment.center,
          height: itemHeight,
          width: itemWidth,
          child: Text(
            sdkLists[index],
            style: TextStyle(
                color: activeTextColor == null ?? defaultActiveTextColor,
                fontSize: widget.fontSize),
          ),
        ),
      ),
    );
  }

  BoxShadow defaultActiveBoxShadow = BoxShadow(
      color: Color(0x44697FFE),
      offset: Offset(0.0, 3.0), //阴影xy轴偏移量
      blurRadius: 6.0, //阴影模糊程度
      spreadRadius: 1.0 //阴影扩散程度
      );

  BoxShadow defaultNormalBoxShadow = BoxShadow(
      color: Color(0x44697FFE),
      offset: Offset(0.0, 3.0), //阴影xy轴偏移量
      blurRadius: 6.0, //阴影模糊程度
      spreadRadius: 1.0 //阴影扩散程度
      );

  Widget roundDefaultItem(int index) {
    return Container(
      margin: EdgeInsets.only(right: widget.itemSpace),
      child: InkWell(
        onTap: () {
          updateSelectIndex(index);
        },
        child: Container(
          height: itemHeight,
          width: itemWidth,
          alignment: Alignment.center,
          decoration: BoxDecoration(
              boxShadow: [normalBoxShadow ?? defaultNormalBoxShadow],
              borderRadius: BorderRadius.circular(widget.radius),
              color: Colors.white),
          child: Text(
            sdkLists[index],
            style: TextStyle(
                color: normalTextColor ?? defaultNormalTextColor,
                fontSize: widget.fontSize),
          ),
        ),
      ),
    );
  }

  Widget lineDefaultItem(int index) {
    return Container(
      margin: EdgeInsets.only(right: widget.itemSpace),
      child: InkWell(
        onTap: () {
          updateSelectIndex(index);
        },
        child: Container(
          alignment: Alignment.center,
          height: itemHeight,
          width: itemWidth,
          child: Text(
            sdkLists[index],
            style: TextStyle(
                color: normalTextColor ?? defaultNormalTextColor,
                fontSize: widget.fontSize),
          ),
        ),
      ),
    );
  }

  Widget lineSelectedItem(int index) {
    return Container(
      margin: EdgeInsets.only(right: widget.itemSpace),
      child: InkWell(
        onTap: () {
          updateSelectIndex(index);
        },
        child: Container(
          decoration: BoxDecoration(
              border: Border(
                  bottom: BorderSide(color: lineColor ?? defaultLineColor))),
          height: itemHeight,
          width: itemWidth,
          alignment: Alignment.center,
          child: Text(
            sdkLists[index],
            style: TextStyle(
                color: activeTextColor ?? defaultActiveTextColor,
                fontSize: widget.fontSize),
          ),
        ),
      ),
    );
  }
}

enum SegmentBarType { round, line }
