import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

///条目开关按钮
typedef OnSwitchChangeListener = void Function(
    String content, int position, bool isChecked);

///定义条目的删除事件监听
typedef OnDeleteListener = void Function(String content, int position);

///定义小图标的点击事件
typedef GestureTapCallback = void Function(int index);

// ignore: must_be_immutable
class ItemActionView extends StatefulWidget {
  //显示的条目数据
  List<String> items;
  List<String> icons;

  //卡片的背景颜色
  Color bgColor;

  //是否具有删除功能
  bool isCanDelete;

  GestureTapCallback onIconTap;

  OnSwitchChangeListener listener;

  OnDeleteListener onDeleteListener;

  //阴影
  double elevation;

  ItemActionView(this.items, this.icons,
      {Key key,
      this.bgColor = Colors.white,
      this.elevation = 3,
      this.onIconTap,
      this.onDeleteListener,
      this.isCanDelete = false,
      this.listener})
      : super(key: key);

  @override
  _ItemActionViewState createState() => _ItemActionViewState();
}

class _ItemActionViewState extends State<ItemActionView> {
  List<bool> _switchValue = [];

  @override
  void initState() {
    super.initState();
    widget.items.forEach((element) {
      _switchValue.add(false);
    });
  }

  @override
  Widget build(BuildContext context) {
    return Card(
      clipBehavior: Clip.antiAlias,
      elevation: widget.elevation,
      shadowColor: Color(0xFFD5D5D5),
      shape: const RoundedRectangleBorder(
        borderRadius: BorderRadius.all(Radius.circular(10)),
      ),
      color: widget.bgColor,
      //颜色
      margin: const EdgeInsets.only(left: 20, right: 20),
      child: widget.items.length == 1
          ? _buildDeleteItem(context, widget.items[0], 0)
          : MediaQuery.removePadding(
              context: context,
              removeTop: true,
              removeBottom: true,
              child: ListView.builder(
                  shrinkWrap: true,
                  //禁用滑动事件
                  physics: new NeverScrollableScrollPhysics(),
                  itemBuilder: (context, index) {
                    return _buildDeleteItem(
                        context, widget.items[index], index);
                  },
                  itemCount: widget.items.length),
            ),
    );
  }

  ///显示对话框进行确认删除
  Future<bool> showTipDialog(BuildContext context) {
    return showCupertinoDialog<bool>(
        barrierDismissible: false,
        builder: (BuildContext context) {
          return CupertinoAlertDialog(
            content: Text("确认删除？"),
            actions: [
              CupertinoDialogAction(
                child: Text('确认'),
                onPressed: () {
                  Navigator.of(context).pop(true);
                },
              ),
              CupertinoDialogAction(
                child: Text('点错了'),
                onPressed: () {
                  Navigator.of(context).pop(false);
                },
              ),
            ],
          );
        },
        context: context);
  }

  Widget _buildDeleteItem(BuildContext context, String content, int index) {
    return widget.isCanDelete
        ? Dismissible(
            confirmDismiss: (direction) {
              //是否确认清除当前控件
              if (direction == DismissDirection.endToStart) {
                return showTipDialog(context);
              }
              return Future<bool>.value(false);
            },
            key: Key("$index$content"),
            direction: DismissDirection.horizontal,
            child: _buildItem(content, index),
            secondaryBackground: Container(
                color: Colors.red,
                alignment: Alignment.centerRight,
                child: Container(
                  margin: EdgeInsets.only(right: 22),
                  child: Text("删除",
                      style: TextStyle(color: Colors.white, fontSize: 14)),
                )),
            background: Container(color: Colors.green),
            onDismissed: (direction) {
              //删除对应位置的数据
              widget.items.removeAt(index);
              _switchValue.removeAt(index);
              if (widget.onDeleteListener != null) {
                widget.onDeleteListener(content, index);
              }
            })
        : _buildItem(content, index);
  }

  Widget _buildItem(String content, int index) {
    return Container(
        child: Column(children: [
      Container(
          padding: EdgeInsets.only(left: 10, right: 10, top: 16, bottom: 16),
          child:
              Row(mainAxisAlignment: MainAxisAlignment.spaceBetween, children: [
            Text(content, style: TextStyle(fontSize: 20, color: Colors.black)),
            Row(
              children: [
                InkWell(
                  onTap: () {
                    if (widget.onIconTap != null) widget.onIconTap(index);
                  },
                  child: Container(
                      clipBehavior: Clip.antiAlias,
                      height: 32,
                      width: 32,
                      decoration: BoxDecoration(
                          borderRadius: BorderRadius.circular(10)),
                      child: Image.asset(widget.icons[index])),
                ),
                Container(
                    margin: EdgeInsets.only(left: 10, right: 10),
                    color: Color(0xFFDBDBDB),
                    height: 28,
                    width: 1),
                CupertinoSwitch(
                  value: _switchValue[index],
                  trackColor: Color(0xFFC6C6C6),
                  activeColor: Color(0xFF34C759),
                  onChanged: (bool value) {
                    _switchValue[index] = value;
                    setState(() {
                      if (widget.listener != null)
                        widget.listener(content, index, value);
                    });
                  },
                ),
              ],
            )
          ])),
      //最后一项不添加间隔条
      index == widget.items.length - 1
          ? SizedBox.shrink()
          : Divider(height: 1, color: Color(0xFFDBDBDB), indent: 10)
    ]));
  }
}
