import 'package:flutter/material.dart';

///定义条目的点击事件
///content条目内容，position条目位置，0开始计算
typedef OnItemClickListener = void Function(String content, int position);

// ignore: must_be_immutable
class ItemAppView extends StatelessWidget {
  //显示的条目数据
  List<String> items;

  //卡片的背景颜色
  Color bgColor;

  OnItemClickListener listener;

  //阴影
  double elevation;

  ItemAppView(this.items,
      {Key key, this.bgColor = Colors.white, this.elevation = 3, this.listener})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Card(
      elevation: elevation,
      shadowColor: Color(0xFFD5D5D5),
      shape: const RoundedRectangleBorder(
        borderRadius: BorderRadius.all(Radius.circular(10)),
      ),
      color: bgColor,
      //颜色
      margin: const EdgeInsets.only(left: 20, right: 20),
      child: items.length == 1
          ? _buildItem(items[0], 0)
          : MediaQuery.removePadding(
              context: context,
              removeTop: true,
              removeBottom: true,
              child: ListView.builder(
                shrinkWrap: true,
                //禁用滑动事件
                physics: NeverScrollableScrollPhysics(),
                itemBuilder: (context, index) {
                  return _buildItem(items[index], index);
                },
                itemCount: items.length,
              ),
            ),
    );
  }

  Widget _buildItem(String content, int index) {
    return InkWell(
      onTap: () {
        if (listener != null) listener(content, index);
      },
      child: Container(
          child: Column(children: [
        Container(
            padding: EdgeInsets.only(left: 10, right: 10, top: 16, bottom: 16),
            child: Row(
                mainAxisAlignment: MainAxisAlignment.spaceBetween,
                children: [
                  Text(content,
                      style: TextStyle(fontSize: 20, color: Colors.black)),
                  Card(
                      elevation: 3,
                      shadowColor: Color(0xFF697FFE),
                      shape: const RoundedRectangleBorder(
                        borderRadius: BorderRadius.all(Radius.circular(20)),
                      ),
                      child: Icon(Icons.chevron_right))
                ])),
        //最后一项不添加间隔条
        index == items.length - 1
            ? SizedBox.shrink()
            : Divider(height: 1, color: Color(0xFFDBDBDB), indent: 10)
      ])),
    );
  }
}
